@@ -1,13 +1,18 @@
 # -*- coding: UTF-8 -*-
+import typing
 from itertools import chain
 from librouteros.protocol import (
     cast_to_api,
 )
+from librouteros.types import (
+    QueryGen,
+    ResponseIter,
+)
 
 
 class Key:
 
-    def __init__(self, name):
+    def __init__(self, name: str):
         self.name = name
 
     def __eq__(self, other):
@@ -23,30 +28,30 @@ class Key:
     def __gt__(self, other):
         yield '?>{}={}'.format(self, cast_to_api(other))
 
-    def __str__(self):
+    def __str__(self) -> str:
         return str(self.name)
 
 
 class Query:
 
-    def __init__(self, path, keys, api):
+    def __init__(self, path, keys: typing.Sequence[Key], api):
         self.path = path
         self.keys = keys
         self.api = api
-        self.query = tuple()
+        self.query: typing.Tuple[str, ...] = tuple()
 
-    def where(self, *args):
+    def where(self, *args: str):
         self.query = tuple(chain.from_iterable(args))
         return self
 
-    def __iter__(self):
+    def __iter__(self) -> ResponseIter:
         keys = ','.join(str(key) for key in self.keys)
         keys = '=.proplist={}'.format(keys)
         cmd = str(self.path.join('print'))
         return iter(self.api.rawCmd(cmd, keys, *self.query))
 
 
-def And(left, right, *rest):
+def And(left: QueryGen, right: QueryGen, *rest: QueryGen) -> QueryGen:
     #pylint: disable=invalid-name
     yield from left
     yield from right
@@ -55,7 +60,7 @@ def And(left, right, *rest):
     yield from ('?#&', ) * len(rest)
 
 
-def Or(left, right, *rest):
+def Or(left: QueryGen, right: QueryGen, *rest: QueryGen) -> QueryGen:
     #pylint: disable=invalid-name
     yield from left
     yield from right
